<?php
/**
 * @brief		Member Points System Application Class
 * @author		<a href='https://community.invisionpower.com/profile/284008-kevin-carwile/'>Kevin Carwile</a>
 * @copyright		(c) 2015 Kevin Carwile
 * @package		IPS Social Suite
 * @subpackage		Member Points System
 * @since		20 Nov 2015
 * @version		
 */
 
namespace IPS\points;

const TRANSACTION_CREDIT = 1;
const TRANSACTION_DEBIT = 2;

const TRANSFER_NONE = 0;
const TRANSFER_BANK = 1;
const TRANSFER_EXCHANGE = 2;
const TRANSFER_MEMBER = 3;

/**
 * Member Points System Application Class
 */
class _Application extends \IPS\Application
{

	/** 
	 * Icon
	 */
	public function get__icon()
	{
		return 'money';
	}
	
	/**
	 * Application Data
	 */
	public function get_appdata()
	{
		return array( 'ver' => $this->version, 'version' => $this->long_version, 'state' => 0, 'url' => \IPS\Settings::i()->base_url );
	}
	
	/**
	 * Install 'other' items. Left blank here so that application classes can override for app
	 *  specific installation needs. Always run as the last step.
	 *
	 * @return void
	 */
	public function installOther()
	{
		static::installNexusGateway();
		parent::installOther();
	}
	
	/**
	 * Install the nexus gateway files
	 *
	 * @return	void
	 */
	public static function installNexusGateway()
	{
		$gatewayDir = \IPS\ROOT_PATH . '/applications/nexus/sources/Gateway';
		$sourceDir = \IPS\ROOT_PATH . '/applications/points/sources/Nexus/Points';
		
		if ( is_dir( $gatewayDir ) )
		{
			mkdir( $gatewayDir .= '/Points' );
			
			copy( $sourceDir . '/Points.php', $gatewayDir . '/Points.php' );
			copy( $sourceDir . '/index.html', $gatewayDir . '/index.html' );
		}
	}
	
	/**
	 * Get URL
	 *
	 * @return	\IPS\Http\Url
	 */
	public function url()
	{
		$args = func_get_args();
		if ( $args[0] == 'update' )
		{
			return \IPS\Http\Url::external( $this->update_check )->setQueryString( array_merge( $this->appdata, array( 'ips_version' => \IPS\Application::load( 'core' )->version ) ) );
		}

		return parent::url();
	}
	
	/**
	 * Install JSON Data
	 */
	public function installJsonData( $skipMember=FALSE )
	{
		/* Update app version data */
		$versions = $this->getAllVersions();
		$lversions = array_keys( $versions );
		$hversions = array_values( $versions );
		$updates = $this->url( 'update' );
		
		if( count($versions) )
		{
			$ver = array_pop( $hversions );
			$version = array_pop( $lversions );
			$updates = $updates->setQueryString( array( 'ver' => $ver, 'version' => $version, 'installed' => 1 ) );
		}
		
		call_user_func_array( 'parent::installJsonData', func_get_args() );
		try { $updates->request()->get(); } catch( \Exception $e ) { }
	}
	
	/**
	 * Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		$this->url( 'update' )->setQueryString( 'installed', 0 )->request()->get();
	}

}
