<?php
/**
 * @brief		Background Task
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	Member Points Economy
 * @since		12 Dec 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\points\extensions\core\Queue;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Background Task
 */
class _BalanceUpdate
{
	/**
	 * Run Background Task
	 *
	 * @param	mixed					$data	Data as it was passed to \IPS\Task::queue()
	 * @param	int						$offset	Offset
	 * @return	int|null				New offset or NULL if complete
	 * @throws	\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
	 */
	public function run( $data, $offset )
	{
		$time = time();
		
		/* Make sure we have enough time to run the updates */
		if ( ini_get( 'max_execution_time' ) < 120 )
		{
			set_time_limit( 120 );
		}
		
		/* Update in batches of 25 for a maximum of 30 seconds */
		while( time() - 30 < $time )
		{
			$balances = new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( '*', 'points_balances', array( 'balance_id>?', $offset ), 'balance_id ASC', 25 ), 'IPS\points\Balance' );
			
			/* Check if we're done */
			if ( $balances->count() == 0 )
			{
				return NULL;
			}
			
			/* Run the batch */
			foreach( $balances as $balance )
			{
				$offset = $balance->id;
				try {
					$balance->accrue();
				}
				catch( \Exception $e ) { }
			}
		}
		
		/* If we're not done, remember where we left off */
		if ( \IPS\Db::i()->select( 'COUNT(*)', 'points_balances', array( 'balance_id>?', $offset ) )->first() )
		{
			return $offset;
		}
		
		return NULL;
	}
	
	/**
	 * Get Progress
	 *
	 * @param	mixed					$data	Data as it was passed to \IPS\Task::queue()
	 * @param	int						$offset	Offset
	 * @return	array( 'text' => 'Doing something...', 'complete' => 50 )	Text explaning task and percentage complete
	 * @throws	\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
	 */
	public function getProgress( $data, $offset )
	{
		$total = \IPS\Db::i()->select( 'COUNT(*)', 'points_balances' )->first();
		$complete = 0;
		
		if ( $total )
		{
			$done = \IPS\Db::i()->select( 'COUNT(*)', 'points_balances', array( 'balance_id<?', $offset ) )->first();
			$complete = round( ( $done / $total ) * 100 );
		}
		
		return array( 'text' => \IPS\Member::loggedIn()->language()->get( 'points_updating_balances' ), 'complete' => $complete );
	}	
}