<?php
/**
 * @brief		Background Task
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	Member Points Economy
 * @since		28 Nov 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\points\extensions\core\Queue;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Background Task
 */
class _RemoveBank
{
	/**
	 * Prepare Task
	 *
	 * @param	array		$data		Data to preprocess
	 * @return	array
	 */
	public function preQueueData( $data )
	{
		$bank_id = $data[ 'bank_id' ];
		
		\IPS\Db::i()->delete( 'points_credits', array( 'credit_bank_id=?', $bank_id ) );
		\IPS\Db::i()->delete( 'points_debits', array( 'debit_bank_id=?', $bank_id ) );
		
		$data[ 'balance_count' ] = \IPS\Db::i()->select( 'COUNT(*)', 'points_balances', array( 'balance_bank_id=?', $bank_id ) )->first();
		
		if ( $data[ 'balance_count' ] == 0 )
		{
			return NULL;
		}
		
		return $data;
	}

	/**
	 * Run Background Task
	 *
	 * @param	mixed					$data	Data as it was passed to \IPS\Task::queue()
	 * @param	int						$offset	Offset
	 * @return	int|null				New offset or NULL if complete
	 * @throws	\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
	 */
	public function run( $data, $offset )
	{
		$bank_id = $data[ 'bank_id' ];
		
		foreach( new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( '*', 'points_balances', array( 'balance_bank_id=? AND balance_id>?', $bank_id, $offset ), 'balance_id ASC', 100 ), 'IPS\points\Balance' ) as $balance )
		{
			$offset = $balance->id;
			$balance->delete();
		}
		
		if ( \IPS\Db::i()->select( 'COUNT(*)', 'points_balances', array( 'balance_bank_id=? AND balance_id>?', $bank_id, $offset ) )->first() > 0 )
		{
			return $offset;
		}
		
		return NULL;
	}
	
	/**
	 * Get Progress
	 *
	 * @param	mixed					$data	Data as it was passed to \IPS\Task::queue()
	 * @param	int						$offset	Offset
	 * @return	array( 'text' => 'Doing something...', 'complete' => 50 )	Text explaning task and percentage complete
	 * @throws	\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
	 */
	public function getProgress( $data, $offset )
	{
		$bank_id = $data[ 'bank_id' ];
		
		$remaining = \IPS\Db::i()->select( 'COUNT(*)', 'points_balances', array( 'balance_bank_id=? AND balance_id>?', $bank_id, $offset ) )->first();
		$complete = (int) ( ( $data[ 'balance_count' ] - $remaining ) / $data[ 'balance_count' ] * 100 );
		
		return array( 'text' => 'Removing bank balances...', 'complete' => $complete );
	}
}