<?php
/**
 * @brief		Points Economy Extension: Banking
 * @package		Member Points Systems
 * @since		14 Dec 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\points\extensions\points\Economies;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Points economy extension: Banks
 */
class _Banking extends \IPS\points\Economy
{
	/**
	 * Economy Alerts
	 * -------------------
	 * Used to supply alerts related to this economy
	 *
	 * @param	\IPS\Member		$member			The member to get alerts for
	 * @return	array
	 * (
	 * 	array(
	 * 		'type' => 'info', 		// ipsMessage type
	 * 		'title'	=> '',			// Title of alert
	 * 		'message' => '',		// Alert HTML Content
	 * 	)
	 * )
	 */
	public function alerts( \IPS\Member $member )
	{
		return array();
	}
	
	/**
	 * Economy Summary
	 * ------------------
	 * This summary is used in the side column of the dashboard
	 *
	 * @param	\IPS\Member		$member			The member to display the summary for
	 * @return	string						HTML Output
	 */
	public function dashSummary( \IPS\Member $member )
	{
		$unbanked = new \IPS\points\Account\Bank( $member );
		$unbanked->icon = 'money';
		
		$summary = '';
		$summary .= $unbanked;
		
		foreach( \IPS\points\Bank::roots( 'use', $member ) as $bank )
		{
			$bank = new \IPS\points\Account\Bank( $member, $bank );
			$summary .= $bank;
		}
		
		return $summary;
	}
	
	/**
	 * Economy Portal
	 * --------------------
	 * Output detailed information in the main area of the dashboard related to this economy
	 *
	 * @param	\IPS\Member		$member			The member to display the portal for
	 * @return	string						HTML Output
	 */
	public function dashPortal( \IPS\Member $member )
	{
		$output = '';
		$charts = '';
		$currencies = \IPS\points\Currency::roots( 'use', $member );
		
		if ( count( $currencies ) > 1 )
		{
			$output .= \IPS\Theme::i()->getTemplate( 'dashboard', 'points', 'front' )->sectionWrapper
			(
				'pie-chart', \IPS\Member::loggedIn()->language()->addToStack( 'points_net_worth_distribution' ),
				$member->netWorthChart()->render( 'PieChart', array
				( 
					'pieSliceText' => 'label', 
					'legend' => array( 'position' => 'labeled' ), 
					'tooltip' => array( 'text' => 'percentage' ) 
				) )
			);
		}
		
		foreach( $currencies as $currency )
		{
			$charts .= $currency->netWorthChart( $member, 'P3M', 'P7D', 'n/j' )->render( 'AreaChart', array
			( 
				'title' => $currency->_title . ': ' . $currency->display( $currency->netWorthAsOf( \IPS\DateTime::create(), $member ) ),
				'animation' => array( 'startup' => TRUE, 'easing' => 'in', 'duration' => 1000 ),
				'titleTextStyle' => array( 'fontSize' => 16, 'bold' => FALSE ),
			) );
		}
		
		if ( isset( $charts ) )
		{
			$output .= \IPS\Theme::i()->getTemplate( 'dashboard', 'points', 'front' )->sectionWrapper( 'history', \IPS\Member::loggedIn()->language()->addToStack( 'points_net_worth_history' ), $charts );
		}
	
		return $output;
	}
	
	/**
	 * Economy Member Profile Output
	 * --------------------
	 * Output overview information for this economy on the main economy homepage
	 *
	 * @param	\IPS\Member		$member			The member to display content for
	 * @return	string						HTML Output
	 */
	public function profileContent( \IPS\Member $member )
	{
		return '';		
	}
	
	/**
	 * Economy Homepage Output
	 * --------------------
	 * Output overview information for this economy on the main economy homepage
	 *
	 * @return	string						HTML Output
	 */
	public function economyHome()
	{
		return \IPS\Theme::i()->getTemplate( 'economies' )->currencyOverview( \IPS\points\Currency::roots( 'view' ) );
	}
	
}