<?php
/**
 * @brief		Rules conversions: Balances
 * @package		Rules for IPS Social Suite
 * @since		11 Dec 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\points\extensions\rules\Conversions;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Rules conversions extension: Balances
 */
class _Balances
{

	/**
	 * Global Arguments
	 *
	 * Let rules know about any global arguments that your app may have. Global arguments
	 * are made available to all rule configurations (conditions/actions), and can also
	 * be used as token replacements on rules forms.
	 *
	 * @return 	array		Array of global arguments
	 */
	public function globalArguments()
	{	
		return array();
	}

	/**
	 * Conversion Map
	 *
	 * Let rules know how to convert objects into different types of arguments.
	 * For example, if an event provides an \IPS\Content object, a conversion map
	 * will tell rules how to derive another possible argument from it (such as the 
	 * content title).
	 *
	 * @return 	array		Conversion map
	 */
	public function conversionMap()
	{
		$map = array
		(
			'IPS\points\Balance' => array
			(
				'Title' => array
				(
					'argtype' => 'string',
					'nullable' => FALSE,
					'converter' => function( $balance )
					{
						return $balance->title;
					},
					'token' => 'title',
					'description' => 'The title',
				),
				'Total Amount' => array
				(
					'argtype' => 'float',
					'nullable' => FALSE,
					'converter' => function( $balance )
					{
						return $balance->amount;
					},
					'token' => 'amount',
					'description' => 'The total amount',
				),
				'Available Amount' => array
				(
					'argtype' => 'float',
					'nullable' => FALSE,
					'converter' => function( $balance )
					{
						return $balance->available;
					},
					'token' => 'available',
					'description' => 'The available amount',
				),
				'Holding Amount' => array
				(
					'argtype' => 'float',
					'nullable' => FALSE,
					'converter' => function( $balance )
					{
						return $balance->holding;
					},
					'token' => 'holding',
					'description' => 'The amount in holding',				
				),
				'Unclaimed Amount' => array
				(
					'argtype' => 'float',
					'nullable' => FALSE,
					'converter' => function( $balance )
					{
						return $balance->unclaimed;
					},
					'token' => 'unclaimed',
					'description' => 'The amount of unclaimed credits',
				),
				'Pending Amount' => array
				(
					'argtype' => 'float',
					'nullable' => FALSE,
					'converter' => function( $balance )
					{
						return $balance->unpaid;
					},		
					'token' => 'pending',
					'description' => 'The amount of pending debits',
				),
				'Member' => array
				(
					'argtype' => 'object',
					'class' => 'IPS\Member',
					'nullable' => FALSE,
					'converter' => function( $balance )
					{
						return $balance->member();
					},
				),
				'Currency' => array
				(
					'argtype' => 'object',
					'class' => 'IPS\points\Currency',
					'nullable' => FALSE,
					'converter' => function( $balance )
					{
						return $balance->currency();
					},
				),
				'Bank' => array
				(
					'argtype' => 'object',
					'class' => 'IPS\points\Bank',
					'nullable' => TRUE,
					'converter' => function( $balance )
					{
						return $balance->bank();
					},
				),
			),
		);
		
		return $map;		
	}
	
}