//<?php

class points_hook_ipsMember extends _HOOK_CLASS_
{

	/**
	 * Get Net Worth Chart
	 *
	 * @param	\IPS\DateTime|NULL		$date		The date for which the net worth chart is calculated or NULL for now
	 * @return	\IPS\points\Chart
	 */
	public function netWorthChart( \IPS\DateTime $date=NULL )
	{
		try
		{
			$date = $date ?: \IPS\DateTime::create();
			$chart = new \IPS\points\Chart;
			
			$chart->addHeader( $this->language()->addToStack( 'points_chart_currency' ), 'string' );
			$chart->addHeader( $this->language()->addToStack( 'points_chart_percentage' ), 'number' );
			
			$total_weight = 0;
			$total_worth = 0;
			$slices = array();
			
			foreach( \IPS\points\Currency::roots( 'use', $this ) as $currency )
			{
				$net_value 	= $currency->netWorthAsOf( $date, $this );
				$net_worth 	= $net_value * ( 1 / $currency->exchange_rate );
				
				$total_weight 	+= abs( $net_worth );
				$total_worth 	+= $net_worth;
				
				$slices[ $currency->id ] = array( $currency, abs( $net_worth ), $net_value );
			}
	
			if ( $total_weight )
			{
				foreach( $slices as $slice )
				{
					$currency 	= $slice[ 0 ];
					$weight 	= $slice[ 1 ];
					$value 		= $slice[ 2 ];
					$percentage 	= round( $weight / $total_weight * 100, 1 );
					
					$chart->addRow( array( $currency->_title . ': ' . $currency->display( $value ), $percentage ) );
				}
			}
			
			/* Store total worth value with chart */
			$chart->totalWorth = $total_worth;
			
			return $chart;
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * Get Balance
	 *
	 * @param	int|\IPS\points\Currency	$currency		Currency of balance or currency id
	 * @param	int|\IPS\points\Bank		$bank			Bank of balance or bank id
	 * @return	\IPS\points\Balance
	 */
	public function balance( $currency, $bank=0 )
	{
		try
		{
			return \IPS\points\Balance::get( $this, $currency, $bank );
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * Check For Balance
	 *
	 * @param	int|\IPS\points\Currency	$currency		Currency of balance or currency id
	 * @param	int|\IPS\points\Bank		$bank			Bank of balance or bank id
	 * @param	bool				$nonZero		Check if the balance is allowed and is also non zero
	 * @return	bool
	 */
	public function hasBalance( $currency, $bank=0, $nonZero=TRUE )
	{
		try
		{
			try
			{
				$balance = \IPS\points\Balance::get( $this, $currency, $bank );
				
				if ( $nonZero )
				{
					return $balance->amount != 0;
				}
				
				return TRUE;
			}
			catch( \IPS\points\Exeception $e ) { }
			
			return FALSE;
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * @brief	Cache for member currencies
	 */
	protected $currencies;
	
	/**
	 * Get Member Currencies
	 *
	 * @return	array
	 */
	public function currencies()
	{
		try
		{
			if ( isset( $this->currencies ) )
			{
				return $this->currencies;
			}
			
			$this->currencies = \IPS\points\Currency::roots( 'use', $this );
			return $this->currencies;
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * Profile Fields shown next to users content
	 *
	 * @return array
	 */
	public function contentProfileFields()
	{
		try
		{
			$contentProfileFields = parent::contentProfileFields();
			$contentProfileFields[ 'points' ] = array();
			
			foreach( $this->currencies() as $currency )
			{
				if ( $currency->can( 'view' ) )
				{
					$worth = $currency->netWorthAsOf( NULL, $this );
					if ( $worth != 0 )
					{
						$contentProfileFields[ 'points' ][] = \IPS\Theme::i()->getTemplate( 'components', 'points', 'front' )->currencyUserbit( $currency, $worth, $this );
					}
				}
			}
			
			return $contentProfileFields;
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	
}