//<?php

class points_hook_ipsNexusPackage extends _HOOK_CLASS_
{

	/**
	 * @brief	Points Settings
	 */
	protected $settingsCache;
	
	/**
	 * Get Points Settings
	 */
	public function get_points_settings()
	{
		try
		{
			if ( isset( $this->settingsCache ) )
			{
				return $this->settingsCache;
			}
			
			try
			{
				$settings = \IPS\Db::i()->select( 'p_points_data', 'points_nexus_packages', array( 'p_id=?', $this->id ) )->first();
				$this->settingsCache = json_decode( $settings, TRUE );
			}
			catch( \UnderflowException $e )
			{
				$this->settingsCache = array();
			}
			
			return $this->settingsCache;
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * Set Points Settings
	 */
	public function set_points_settings( $data )
	{
		try
		{
			$this->settingsCache = $data;
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		try
		{
			parent::form( $form );
			
			$points = $this->points_settings;
			$currencies = \IPS\points\Currency::roots( NULL );
			if ( count( $currencies ) )
			{
				$form->addTab( 'points_nexus_credits' );
				$form->addMessage( 'points_nexus_credits_info' );
				foreach( $currencies as $currency )
				{
					$form->add( new \IPS\Helpers\Form\Number( 'points_currency_' . $currency->id, $points[ 'points_' . $currency->id ] ?: 0, FALSE, array( 'min' => 0, 'decimals' => $currency->decimals ), NULL, NULL, NULL, 'points_currency_' . $currency->id ) );
				}
			}
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}

	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function saveForm( $values )
	{
		try
		{
			$points_settings = array();
			
			foreach( $values as $key => $value )
			{
				if ( mb_substr( $key, 0, 16 ) == 'points_currency_' )
				{
					$points_settings[ 'points_' . mb_substr( $key, 16 ) ] = $value;
				}
			}
			
			$this->points_settings = $points_settings;
			
			return parent::saveForm( $values );
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * Credit points for purchase
	 *
	 * @param	\IPS\nexus\Invoice		$invoice		The associated invoice
	 * @param	\IPS\nexus\Invoice\Item		$line			The associated invoice item
	 * @return	void
	 */
	public function creditPoints( \IPS\nexus\Invoice $invoice, \IPS\nexus\Invoice\Item $line )
	{
		try
		{
			if ( $points = $this->points_settings )
			{	
				foreach( \IPS\points\Currency::roots( NULL ) as $currency )
				{
					if ( isset( $points[ 'points_' . $currency->id ] ) and $points[ 'points_' . $currency->id ] > 0 )
					{
						$credit_amount = $line->quantity * $points[ 'points_' . $currency->id ];
						
						try
						{
							$balance = \IPS\points\Balance::get( $invoice->member, $currency );
							$balance->credit( $credit_amount, "{$line->name} x{$line->quantity} (#{$invoice->id})", 0, array(), FALSE );
						}
						catch( \IPS\points\Exception $e ) { }
						catch( \OutOfRangeException $e ) { }
					}
				}
			}
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}

	/**
	 * Debit points for cancelled purchase
	 *
	 * @param	\IPS\nexus\Invoice		$invoice		The associated invoice
	 * @param	\IPS\nexus\Invoice\Item		$line			The associated invoice item
	 * @return	void
	 */
	public function debitPoints( \IPS\nexus\Invoice $invoice, \IPS\nexus\Invoice\Item $line )
	{
		try
		{
			if ( $points = $this->points_settings )
			{	
				foreach( \IPS\points\Currency::roots( NULL ) as $currency )
				{
					if ( isset( $points[ 'points_' . $currency->id ] ) and $points[ 'points_' . $currency->id ] > 0 )
					{
						$debit_amount = $line->quantity * $points[ 'points_' . $currency->id ];
						
						try
						{
							$balance = \IPS\points\Balance::get( $invoice->member, $currency );
							$balance->debit( $debit_amount, "{$line->name} x{$line->quantity} (#{$invoice->id})", 0, array(), FALSE );
						}
						catch( \IPS\points\Exception $e ) { }
						catch( \OutOfRangeException $e ) { }
					}
				}
			}
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}

	/**
	 * [ActiveRecord] Save Record
	 *
	 * @return	void
	 */
	public function save()
	{
		try
		{
			parent::save();
			
			if ( isset( $this->settingsCache ) )
			{
				if ( \IPS\Db::i()->select( 'COUNT(*)', 'points_nexus_packages', array( 'p_id=?', $this->id ) )->first() )
				{
					\IPS\Db::i()->update( 'points_nexus_packages', array( 'p_id' => $this->id, 'p_points_data' => json_encode( $this->settingsCache ) ), array( 'p_id=?', $this->id ) );
				} 
				else
				{
					\IPS\Db::i()->insert( 'points_nexus_packages', array( 'p_id' => $this->id, 'p_points_data' => json_encode( $this->settingsCache ) ) );
				}
			}
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		try
		{
			parent::delete();
			\IPS\Db::i()->delete( 'points_nexus_packages', array( 'p_id=?', $this->id ) );
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	

}