<?php


namespace IPS\points\modules\admin\points;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * balances
 */
class _balances extends \IPS\Dispatcher\Controller
{	
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'balances_manage' );
		parent::execute();
	}
	
	/**
	 * Manage
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$bank_id 	= is_numeric( \IPS\Request::i()->tab ) ? \IPS\Request::i()->tab : 0;
		$tabs 		= array( 0 => 'points_unbanked' );
		$active_tab 	= $bank_id ?: 'unbanked';
		$url		= \IPS\Http\Url::internal( 'app=points&module=points&controller=balances' );
		
		$table = \IPS\points\Bank::balancesTable( $bank_id, $url->setQueryString( array( 'tab' => $bank_id ) ) );
		
		/* Set default filter */
		if ( \IPS\Request::i()->filter == 'default' or \IPS\Request::i()->tab === NULL )
		{
			$keys = array_keys( $table->filters );
			$table->filter = array_pop( $keys );
		}
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->output = $table;
			return;
		}
		
		foreach( \IPS\points\Bank::roots() as $bank )
		{
			$tabs[ $bank->id ] = $bank->_title;
		}
		
		$output = \IPS\Theme::i()->getTemplate( 'global', 'core' )->tabs( $tabs, $active_tab, (string) $table, $url->setQueryString( array( 'filter' => 'default' ) ) );
		
		/* Display */
		\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack( 'points_balances' );
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'global', 'core' )->block( 'title', $output );
	}
	
	/**
	 * Balance Transactions
	 */
	protected function transactions()
	{
		$balance = \IPS\points\Balance::load( \IPS\Request::i()->balance_id );
		
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack( 'points_transactions' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'global', 'core' )->block( 'title', $balance->transactionsTable( 25, \IPS\Http\Url::internal( 'app=points&module=points&controller=balances&do=transactions' )->setQueryString( 'balance_id', $balance->id ) ) );
	}
	
	/**
	 * Balance Adjustment
	 */
	protected function adjustment()
	{
		$balance = NULL;
		if ( \IPS\Request::i()->balance_id )
		{
			$balance = \IPS\points\Balance::load( \IPS\Request::i()->balance_id );
		}
		
		$form = new \IPS\Helpers\Form( 'points_adjustment', 'points_issue' );
		$form->class = 'ipsPad ipsForm_horizontal';
		
		if ( $balance )
		{
			$form->addHtml( \IPS\Theme::i()->getTemplate( 'components', 'points', 'front' )->balanceOverview( $balance ) );
		}
		
		$adjustment_options = array
		(
			'credit' => 'transaction_credit',
			'debit' => 'transaction_debit',
			'transfer' => 'transaction_transfer',
		);
		
		$adjustment_toggles = array
		(
			'transfer' => array( 'points_transfer_member', 'points_transfer_currency', 'points_transfer_bank', 'points_adjustment_header_points_destination' ),
		);
		
		if ( $balance )
		{
			$form->hiddenValues[ 'points_adjustment_member' ] = $balance->member_id;
			$form->hiddenValues[ 'points_adjustment_currency' ] = $balance->currency_id;
			$form->hiddenValues[ 'points_adjustment_bank' ] = $balance->bank_id;
		}
		else
		{
			$form->addHeader( 'points_source' );
			$form->add( new \IPS\Helpers\Form\Member( 'points_adjustment_member', NULL, TRUE ) );
			$form->add( new \IPS\Helpers\Form\Node( 'points_adjustment_currency', NULL, TRUE, array( 'class' => 'IPS\points\Currency' ) ) );
			$form->add( new \IPS\Helpers\Form\Node( 'points_adjustment_bank', (int) \IPS\Request::i()->bank_id, TRUE, array( 'zeroVal' => 'points_unbanked', 'class' => 'IPS\points\Bank' ) ) );
		}
		
		$form->add( new \IPS\Helpers\Form\Radio( 'points_adjustment_method', 'credit', TRUE, array( 'options' => $adjustment_options, 'toggles' => $adjustment_toggles ) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'points_adjustment_amount', 0, TRUE, array( 'min' => 0, 'decimals' => ( $balance ? $balance->currency()->decimals : 5 ) ) ) );
		$form->add( new \IPS\Helpers\Form\Text( 'points_adjustment_message', '', FALSE ) );
		$form->add( new \IPS\Helpers\Form\Date( 'points_adjustment_expiration', 0, FALSE, array( 'unlimited' => 0, 'unlimitedLang' => 'points_adjustment_expiration_none', 'time' => TRUE ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'points_adjustment_force', FALSE, FALSE ) );
		
		$transfer_required = ( \IPS\Request::i()->points_adjustment_submitted and \IPS\Request::i()->points_adjustment_method == 'transfer' );

		$form->addHeader( 'points_destination' );
		$form->add( new \IPS\Helpers\Form\Member( 'points_transfer_member', $balance ? $balance->member()->name : NULL, $transfer_required, array(), NULL, NULL, NULL, 'points_transfer_member' ) );
		$form->add( new \IPS\Helpers\Form\Node( 'points_transfer_currency', $balance ? $balance->currency_id : NULL, $transfer_required, array( 'class' => 'IPS\points\Currency' ), NULL, NULL, NULL, 'points_transfer_currency' ) );
		$form->add( new \IPS\Helpers\Form\Node( 'points_transfer_bank', (int) \IPS\Request::i()->bank_id, $transfer_required, array( 'zeroVal' => 'points_unbanked', 'class' => 'IPS\points\Bank' ), NULL, NULL, NULL, 'points_transfer_bank' ) );
		
		if ( $values = $form->values() )
		{
			try
			{
				$enforcePerms = ! $values[ 'points_adjustment_force' ];
				
				try
				{
					$balance = \IPS\points\Balance::get( $values[ 'points_adjustment_member' ], $values[ 'points_adjustment_currency' ], $values[ 'points_adjustment_bank' ], $enforcePerms );
				}
				catch( \IPS\points\Exception $e )
				{
					if ( $values[ 'points_adjustment_method' ] == 'transfer' )
					{
						$message = \IPS\Member::loggedIn()->language()->get( 'points_source' ) . ': ' . $e->getMessage();
					}
					
					throw new \IPS\points\Exception( $message );
				}
				
				if ( $values[ 'points_adjustment_expiration' ] instanceof \IPS\DateTime )
				{
					$expiration = $values[ 'points_adjustment_expiration' ]->getTimestamp();
				}
				else
				{
					$expiration = 0;
				}
				
				switch( $values[ 'points_adjustment_method' ] )
				{
					case 'credit' :
					
						$balance->credit( $values[ 'points_adjustment_amount' ], $values[ 'points_adjustment_message' ], $expiration, array(), $enforcePerms );
						break;
						
					case 'debit' :
					
						$balance->debit( $values[ 'points_adjustment_amount' ], $values[ 'points_adjustment_message' ], $expiration, array(), $enforcePerms );
						break;
						
					case 'transfer' :
					
						try
						{
							$destination = \IPS\points\Balance::get( $values[ 'points_transfer_member' ], $values[ 'points_transfer_currency' ], $values[ 'points_transfer_bank' ], $enforcePerms );
						}
						catch( \IPS\points\Exception $e )
						{
							$message = \IPS\Member::loggedIn()->language()->get( 'points_destination' ) . ': ' . $e->getMessage();
							throw new \IPS\points\Exception( $message );
						}
						$balance->transfer( $values[ 'points_adjustment_amount' ], $destination, $values[ 'points_adjustment_message' ], $expiration, array(), $enforcePerms );
						break;
				
				}
				
				$bank_id = $balance->bank() ? $balance->bank()->id : 0;
				\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=points&module=points&controller=balances' )->setQueryString( array( 'tab' => $bank_id, 'filter' => 'points_currency_' . $balance->currency()->id ) ), 'points_transaction_complete' );
			}
			catch( \IPS\points\Exception $e )
			{
				$form->error = $e->getMessage();
			}
		}
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'points_adjust_balance' );
		\IPS\Output::i()->output = $form;
		
	}
	
	/** 
	 * Delete Balance
	 */
	protected function delete()
	{
		\IPS\Session::i()->csrfCheck();
		
		$balance = \IPS\points\Balance::load( \IPS\Request::i()->balance_id );
		$bank_id = $balance->bank() ? $balance->bank()->id : 0;
		$currency_id = $balance->currency()->id;
		$balance->delete();
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( \IPS\Member::loggedIn()->language()->get( 'points_balance_deleted' ) );
		}
		
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=points&module=points&controller=balances' )->setQueryString( array( 'tab' => $bank_id, 'filter' => 'points_currency_' . $currency_id ) ), 'points_balance_deleted' );
	}
	
}