<?php


namespace IPS\points\modules\front\points;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * dashboard
 */
class _dashboard extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
	}

	/**
	 * View Main Dashboard
	 *
	 * @return	void
	 */
	protected function manage()
	{	
		$economies = \IPS\Application::allExtensions( 'points', 'Economies' );
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'points_dashboard_title' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'dashboard' )->mainDashboard( $economies );
		
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'points_dashboard_title' ) );
	}
	
	/**
	 * View Balance Dashboard
	 *
	 * @return	void
	 */
	protected function balanceDashboard()
	{
		$balance = \IPS\points\Balance::get( \IPS\Member::loggedIn(), \IPS\Request::i()->curr, \IPS\Request::i()->bank );
		
		\IPS\Output::i()->title = $balance->title;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'dashboard', 'points', 'front' )->balanceDashboard( $balance ); 
		
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=points&module=points&controller=dashboard' ), \IPS\Member::loggedIn()->language()->addToStack( 'points_dashboard_title' ) );
		\IPS\Output::i()->breadcrumb[] = array( NULL, $balance->title );
	}
	
	/**
	 * View Transactions
	 *
	 * @return	void
	 */
	protected function transactions()
	{
		$balance = \IPS\points\Balance::get( \IPS\Member::loggedIn(), \IPS\Request::i()->curr, \IPS\Request::i()->bank );
		$table = (string) $balance->transactionsTable();
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'points_transactions' ) . ': ' . $balance->title;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'dashboard', 'points', 'front' )->viewTransactions( $balance, $table );
		
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=points&module=points&controller=dashboard' ), \IPS\Member::loggedIn()->language()->addToStack( 'points_dashboard_title' ) );
		\IPS\Output::i()->breadcrumb[] = array( $balance->url(), $balance->title );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'points_transactions' ) );
	}
	
	/**
	 * View Unclaimed Credits
	 *
	 * @return	void
	 */
	protected function unclaimed()
	{
		$balance = \IPS\points\Balance::get( \IPS\Member::loggedIn(), \IPS\Request::i()->curr, \IPS\Request::i()->bank );
		$table = (string) $balance->unclaimedTable();
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'points_unclaimed' ) . ': ' . $balance->title;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'dashboard', 'points', 'front' )->viewUnclaimed( $balance, $table );
		
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=points&module=points&controller=dashboard' ), \IPS\Member::loggedIn()->language()->addToStack( 'points_dashboard_title' ) );
		\IPS\Output::i()->breadcrumb[] = array( $balance->url(), $balance->title );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'points_unclaimed' ) );
	}
	
	/**
	 * View Unclaimed Credits
	 *
	 * @return	void
	 */
	protected function unpaid()
	{
		$balance = \IPS\points\Balance::get( \IPS\Member::loggedIn(), \IPS\Request::i()->curr, \IPS\Request::i()->bank );
		$table = (string) $balance->unpaidTable();
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'points_unpaid' ) . ': ' . $balance->title;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'dashboard', 'points', 'front' )->viewUnpaid( $balance, $table );
		
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=points&module=points&controller=dashboard' ), \IPS\Member::loggedIn()->language()->addToStack( 'points_dashboard_title' ) );
		\IPS\Output::i()->breadcrumb[] = array( $balance->url(), $balance->title );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'points_unpaid' ) );
	}
	
	/**
	 * Transfer Points
	 *
	 * @return	void
	 */
	protected function transfer()
	{
		$balance = \IPS\points\Balance::get( \IPS\Member::loggedIn(), \IPS\Request::i()->curr, \IPS\Request::i()->bank );	
		$form = new \IPS\Helpers\Form( 'points_dashboard_transfer', 'points_transfer' );
		$form->class = "ipsPad";
		
		$transfer_options = $balance->bank() ? array( 0 => \IPS\Member::loggedIn()->language()->addToStack( 'points_unbanked' ) ) : array();
		$balanceBankId = $balance->bank() ? $balance->bank()->id : 0;
		
		foreach( \IPS\points\Bank::roots( 'transfer_in', $balance->member() ) as $bank )
		{
			if ( $bank->canUse( $balance->currency(), $balance->member() ) and $bank->id !== $balanceBankId )
			{
				$transfer_options[ $bank->id ] = $bank->_title;
			}
		}
		
		$form->addHtml( \IPS\Theme::i()->getTemplate( 'components', 'points', 'front' )->balanceOverview( $balance ) );
		
		$form->add( new \IPS\Helpers\Form\Number( 'points_dash_transfer_amount', 0, TRUE, array( 'min' => 0, 'decimals' => $balance->currency()->decimals ) ) );
		$form->add( new \IPS\Helpers\Form\Radio( 'points_dash_transfer_bank', NULL, TRUE, array( 'options' => $transfer_options ) ) );
		$form->add( new \IPS\Helpers\Form\Text( 'points_dash_transfer_message', NULL, FALSE ) );
		
		if ( $values = $form->values() )
		{
			try
			{
				$destination = \IPS\points\Balance::get( $balance->member(), $balance->currency(), $values[ 'points_dash_transfer_bank' ] );
				$balance->transfer( $values[ 'points_dash_transfer_amount' ], $destination, $values[ 'points_dash_transfer_message' ] );
				
				$redirect = \IPS\Request::i()->bal ? $balance->url() : \IPS\Http\Url::internal( 'app=points&module=points&controller=dashboard' );
				\IPS\Output::i()->redirect( $redirect, 'points_transfer_complete' );
			}
			catch( \IPS\points\Exception $e )
			{
				$form->error = $e->getMessage();
			}
		}
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'points_transfer_amount' );
		\IPS\Output::i()->output = $form;
		
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=points&module=points&controller=dashboard' ), \IPS\Member::loggedIn()->language()->addToStack( 'points_dashboard_title' ) );
		\IPS\Output::i()->breadcrumb[] = array( $balance->url(), $balance->title );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'points_transfer_amount' ) );
	}
	
	/**
	 * Exchange Points
	 *
	 * @return	void
	 */
	protected function exchange()
	{
		$balance = \IPS\points\Balance::get( \IPS\Member::loggedIn(), \IPS\Request::i()->curr, \IPS\Request::i()->bank );	
		$form = new \IPS\Helpers\Form( 'points_dashboard_exchange', 'points_exchange' );
		$form->class = "ipsPad";
		
		$exchange_options = array();
		
		$currencies = $balance->bank() ? $balance->bank()->currencies( 'exchange_in', $balance->member() ) : \IPS\points\Currency::roots( 'exchange_in', $balance->member() );
		
		foreach( $currencies as $currency )
		{
			if ( $currency->id !== $balance->currency()->id )
			{
				$exchange_options[ $currency->id ] = $currency->_title;
			}
		}
		
		$form->addHtml( \IPS\Theme::i()->getTemplate( 'components', 'points', 'front' )->balanceOverview( $balance ) );
		
		$form->add( new \IPS\Helpers\Form\Number( 'points_dash_transfer_amount', 0, TRUE, array( 'min' => 0, 'decimals' => $balance->currency()->decimals ) ) );
		$form->add( new \IPS\Helpers\Form\Radio( 'points_dash_transfer_currency', NULL, TRUE, array( 'options' => $exchange_options ) ) );
		$form->add( new \IPS\Helpers\Form\Text( 'points_dash_transfer_message', NULL, FALSE ) );
		
		if ( $values = $form->values() )
		{
			try
			{
				$destination = \IPS\points\Balance::get( $balance->member(), \IPS\points\Currency::load( $values[ 'points_dash_transfer_currency' ] ), $balance->bank() );
				$balance->transfer( $values[ 'points_dash_transfer_amount' ], $destination, $values[ 'points_dash_transfer_message' ] );
				
				$redirect = \IPS\Request::i()->bal ? $balance->url() : \IPS\Http\Url::internal( 'app=points&module=points&controller=dashboard' );
				\IPS\Output::i()->redirect( $redirect, 'points_exchange_complete' );
			}
			catch( \IPS\points\Exception $e )
			{
				$form->error = $e->getMessage();
			}
		}
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'points_exchange_amount' );
		\IPS\Output::i()->output = $form;
		
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=points&module=points&controller=dashboard' ), \IPS\Member::loggedIn()->language()->addToStack( 'points_dashboard_title' ) );
		\IPS\Output::i()->breadcrumb[] = array( $balance->url(), $balance->title );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'points_exchange_amount' ) );
	}
	
	/**
	 * Send Points
	 *
	 * @return	void
	 */
	protected function send()
	{
		$balance = \IPS\points\Balance::get( \IPS\Member::loggedIn(), \IPS\Request::i()->curr, \IPS\Request::i()->bank );	
		$form = new \IPS\Helpers\Form( 'points_dashboard_send', 'points_send' );
		$form->class = "ipsPad";
		
		$exchange_options = array();
		
		$form->addHtml( \IPS\Theme::i()->getTemplate( 'components', 'points', 'front' )->balanceOverview( $balance ) );
		
		$form->add( new \IPS\Helpers\Form\Number( 'points_dash_transfer_amount', 0, TRUE, array( 'min' => 0, 'decimals' => $balance->currency()->decimals ) ) );
		$form->add( new \IPS\Helpers\Form\Member( 'points_dash_transfer_member', NULL, TRUE ) );
		$form->add( new \IPS\Helpers\Form\Text( 'points_dash_transfer_message', NULL, FALSE ) );
		
		if ( $values = $form->values() )
		{
			try
			{
				$destination = \IPS\points\Balance::get( $values[ 'points_dash_transfer_member' ], $balance->currency() );
				$balance->transfer( $values[ 'points_dash_transfer_amount' ], $destination, $values[ 'points_dash_transfer_message' ] );
				
				$redirect = \IPS\Request::i()->bal ? $balance->url() : \IPS\Http\Url::internal( 'app=points&module=points&controller=dashboard' );
				\IPS\Output::i()->redirect( $redirect, 'points_send_complete' );
			}
			catch( \IPS\points\Exception $e )
			{
				$form->error = $e->getMessage();
			}
		}
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'points_send_to_member' );
		\IPS\Output::i()->output = $form;	
		
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=points&module=points&controller=dashboard' ), \IPS\Member::loggedIn()->language()->addToStack( 'points_dashboard_title' ) );
		\IPS\Output::i()->breadcrumb[] = array( $balance->url(), $balance->title );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'points_send_to_member' ) );
	}
	
	/**
	 * Claim Credits
	 * 
	 * @return	void
	 */
	protected function claim()
	{
		$credit = \IPS\points\Transaction\Credit::load( \IPS\Request::i()->credit );
		
		if ( \IPS\Member::loggedIn()->member_id !== $credit->member()->member_id )
		{
			\IPS\Output::i()->error( 'points_unauthorized_action', '2P300/1', 403, ''  );
		}
		
		try
		{
			$credit->issue( $credit->message );
			\IPS\Output::i()->redirect( $credit->balance()->url(), 'points_credit_claimed' );
		}
		catch( \IPS\points\Exception $e )
		{
			\IPS\Output::i()->error( $e->getMessage(), '2P301/1', 403, ''  );
		}
	}
	
	/**
	 * Pay Pending Debits
	 *
	 * @return	void
	 */
	protected function pay()
	{
		$debit = \IPS\points\Transaction\Debit::load( \IPS\Request::i()->debit );
		
		if ( \IPS\Member::loggedIn()->member_id !== $debit->member()->member_id )
		{
			\IPS\Output::i()->error( 'points_unauthorized_action', '2P300/2', 403, '' );
		}
		
		try
		{
			$debit->issue( $debit->message );
			\IPS\Output::i()->redirect( $debit->balance()->url(), 'points_debit_paid' );
		}
		catch( \IPS\points\Exception $e )
		{
			\IPS\Output::i()->error( $e->getMessage(), '2P301/2', 403, ''  );
		}		
	}
		
}