<?php
/**
 * @brief		Member Points Systems
 * @author		Kevin Carwile (http://www.linkedin.com/in/kevincarwile)
 * @copyright		(c) 2015 - Kevin Carwile
 * @package		Points
 * @since		19 Nov 2015
 */

namespace IPS\points\Account;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Points System Configuration
 */
class _Bank extends \IPS\points\Account
{
	/**
	 * @brief	Icon
	 */
	public $icon = 'institution';
		
	/**
	 * @brief	Member
	 */
	protected $member;
	
	/**
	 * @brief	Bank
	 */
	protected $bank;
	
	/**
	 * [Constructor]
	 */
	public function __construct( $member, $bank=NULL )
	{
		$this->member = $member;
		$this->bank = $bank;
	}
	
	/**
	 * Get Icon
	 *
	 * @return	string		HTML Output
	 */
	public function icon()
	{
		return "<i class='fa fa-{$this->icon}'></i>";
	}
	
	/**
	 * Get Title
	 *
	 * @return	string		Account Title
	 */
	public function title()
	{
		return $this->bank ? $this->bank->_title : \IPS\Member::loggedIn()->language()->addToStack( 'points_unbanked' );
	}
	
	/**
	 * Get Summary
	 *
	 * @return	string
	 */
	public function summary()
	{
		$dashboard = \IPS\Theme::i()->getTemplate( 'dashboard', 'points', 'front' );
		$output = '';
		
		if ( $this->bank )
		{
			foreach( $this->bank->balances( $this->member ) as $balance )
			{
				$output .= $dashboard->balanceItem( $balance );
			}
		}
		else
		{
			foreach( \IPS\points\Currency::roots( 'use', $this->member ) as $currency )
			{
				$balance = \IPS\points\Balance::get( $this->member, $currency );
				$output .= $dashboard->balanceItem( $balance );
			}
		}
		
		return $output;
	}
	
}
