<?php
/**
 * @brief		Member Points Systems
 * @author		Kevin Carwile (http://www.linkedin.com/in/kevincarwile)
 * @copyright		(c) 2015 - Kevin Carwile
 * @package		Points
 * @since		19 Nov 2015
 */

namespace IPS\points;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Points System Configuration
 */
class _Bank extends \IPS\Node\Model implements \IPS\Node\Permissions
{

	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;
	
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'points_banks';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'bank_';
		
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'weight';
	
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'points_banks';
	
	/**
	 * @brief	[Node] App for permission index
	 */
	public static $permApp = 'points';
	
	/**
	 * @brief	[Node] Type for permission index
	 */
	public static $permType = 'points_bank';
	
	/**
	 * @brief	The map of permission columns
	 */
	public static $permissionMap = array
	(
		'view'			=> 'view',
		'use'			=> 2,
		'transfer_in'		=> 3,
		'transfer_out'		=> 4,
	);
	
	/**
	 * @brief	[Node] Prefix string that is automatically prepended to permission matrix language strings
	 */
	public static $permissionLangPrefix = 'points_bank_';

	/**
	 *  Disable Copy Button
	 */	
	public $noCopyButton = TRUE;
	
	/**
	 * Get Title
	 */
	public function get__title()
	{
		return $this->name;
	}
	
	/**
	 * Get bank data
	 */
	public function get_bank_data()
	{
		return json_decode( $this->data, TRUE ) ?: array();
	}
	 
	/**
	 * Set bank data
	 */
	public function set_bank_data( $data )
	{
		$this->data = json_encode( $data );
	}
	
	/**
	 * Form
	 */
	public function form( &$form )
	{
		$lang = \IPS\Member::loggedIn()->language();
	
		$form->add( new \IPS\Helpers\Form\Text( 'bank_name', $this->name, TRUE ) );
		$form->add( new \IPS\Helpers\Form\Translatable( 'bank_description', NULL, FALSE, array
		(
			'app'		=> 'points',
			'key'		=> ( $this->id ? "bank_description_{$this->id}" : NULL ),
			'editor'	=> array(
				'app'			=> 'points',
				'key'			=> 'Generic',
				'autoSaveKey'		=> ( $this->id ? "bank-desc-{$this->id}" : "bank-new-desc" ),
				'attachIds'		=> $this->id ? array( $this->id, NULL, 'bank_description' ) : NULL, 
				'minimize'		=> 'bank_desc_placeholder'
			)
		) ) );
		
		$currencies = \IPS\points\Currency::roots( NULL );
		$bank_data = $this->bank_data;
		$currency_options = array();
		$currency_toggles = array();
		
		foreach( $currencies as $currency )
		{
			$currency_options[ $currency->id ] = $currency->_title;
			$currency_toggles[ $currency->id ] = array
			( 
				$form->id . '_header_' . md5( $currency->_title . $currency->id ), 
				'bank_currency_minimum_' . $currency->id, 
				'bank_currency_maximum_' . $currency->id,
				'bank_currency_holding_' . $currency->id,
				'bank_currency_interest_' . $currency->id,
			);
		}
		
		$form->add( new \IPS\Helpers\Form\CheckboxSet( 'bank_currencies', explode( ',', $this->currencies ) ?: array_keys( $currency_options ), TRUE, array( 'options' => $currency_options, 'toggles' => $currency_toggles ) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'bank_cycle_interval', $this->cycle_interval ?: 7, TRUE, array( 'min' => 1, 'max' => 999 ), NULL, NULL, ' days' ) );
		
		foreach( $currencies as $currency )
		{
			$lang->words[ md5( $currency->_title . $currency->id ) ] = $currency->_title;
			$form->addHeader( md5( $currency->_title . $currency->id ) );
			
			$lang->words[ 'bank_currency_minimum_' . $currency->id ] = $lang->get( 'bank_currency_minimum' );
			$lang->words[ 'bank_currency_maximum_' . $currency->id ] = $lang->get( 'bank_currency_maximum' );
			$lang->words[ 'bank_currency_holding_' . $currency->id ] = $lang->get( 'bank_currency_holding' );
			$lang->words[ 'bank_currency_interest_' . $currency->id ] = $lang->get( 'bank_currency_interest' );
			
			$form->add( new \IPS\Helpers\Form\Number( 'bank_currency_minimum_' . $currency->id, isset( $bank_data[ 'bank_currency_minimum_' . $currency->id ] ) ? $bank_data[ 'bank_currency_minimum_' . $currency->id ] : 0, TRUE, array( 'min' => NULL, 'decimals' => $currency->decimals ), NULL, $currency->prefix, $currency->suffix, 'bank_currency_minimum_' . $currency->id ) );
			$form->add( new \IPS\Helpers\Form\Number( 'bank_currency_maximum_' . $currency->id, isset( $bank_data[ 'bank_currency_maximum_' . $currency->id ] ) ? $bank_data[ 'bank_currency_maximum_' . $currency->id ] : -9999, TRUE, array( 'min' => NULL, 'decimals' => $currency->decimals, 'unlimited' => -9999, 'unlimitedLang' => 'bank_unlimited_maximum' ), NULL, $currency->prefix, $currency->suffix, 'bank_currency_maximum_' . $currency->id ) );
			$form->add( new \IPS\Helpers\Form\Number( 'bank_currency_holding_' . $currency->id, isset( $bank_data[ 'bank_currency_holding_' . $currency->id ] ) ? $bank_data[ 'bank_currency_holding_' . $currency->id ] : 0, TRUE, array( 'min' => 0, 'decimals' => 3 ), NULL, NULL, ' days', 'bank_currency_holding_' . $currency->id ) );
			$form->add( new \IPS\Helpers\Form\Number( 'bank_currency_interest_' . $currency->id, isset( $bank_data[ 'bank_currency_interest_' . $currency->id ] ) ? $bank_data[ 'bank_currency_interest_' . $currency->id ] : 0, TRUE, array( 'min' => 0, 'max' => 100, 'decimals' => 3 ), NULL, NULL, '% (daily)', 'bank_currency_interest_' . $currency->id ) );
		}
		
		parent::form( $form );
	}
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function saveForm( $values )
	{
	
		$is_new = ! $this->id;
		$bank_data = $this->bank_data;
		
		if ( is_array( $values[ 'bank_currencies' ] ) )
		{
			$currencies = array();
			foreach( $values[ 'bank_currencies' ] as $currency )
			{
				$currencies[] = ( $currency instanceof \IPS\points\Currency ) ? $currency->id : $currency;
			}
			$values[ 'bank_currencies' ] = implode( ',', $currencies );
		}
		
		if ( $values[ 'bank_fee_currency' ] instanceof \IPS\points\Currency )
		{
			$values[ 'bank_fee_currency' ] = $values[ 'bank_fee_currency' ]->id;
		}
		
		/* Update bank data */
		foreach ( $values as $key => $value )
		{
			if ( mb_substr( $key, 0, 14 ) == 'bank_currency_' )
			{
				$bank_data[ $key ] = $value;
				unset( $values[ $key ] );
			}
		}
		
		$bank_description = $values[ 'bank_description' ];
		unset( $values[ 'bank_description' ] );
		
		/* Set bank data */
		$this->bank_data = $bank_data;
		
		parent::saveForm( $values );
		
		/* Save description */
		\IPS\Lang::saveCustom( 'points', "bank_description_{$this->id}", $bank_description );
		
		/* Claim attachments */
		if ( $is_new )
		{
			\IPS\File::claimAttachments( 'bank-new-desc', $this->id, NULL, 'description', TRUE );
		}
		else
		{
			\IPS\File::claimAttachments( 'bank-desc-' . $this->id, $this->id, NULL, 'description', TRUE );
		}
			
	}
	
	/**
	 * [Node] Does the currently logged in user have permission to copy this node?
	 *
	 * @return	bool
	 */
	public function canCopy()
	{
		return FALSE;
	}
	
	/**
	 * Can Use?
	 *
	 * @param	\IPS\points\Currency	$currency	The currency to check if member can use
	 * @param	\IPS\Member|NULL	$member		The member to check or NULL for current logged in member
	 * @return	bool
	 */
	public function canUse( $currency, $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		
		if ( $this->currencies == 0 )
		{
			$currency_available = TRUE;
		}
		else
		{
			$currency_available = in_array( $currency->id, explode( ',', $this->currencies ) );
		}
		
		return $currency->can( 'use', $member ) and $this->can( 'use', $member ) and $currency_available;
	}
	
	/** 
	 * Bank Currencies
	 *
	 * @param	string|NULL		$perm		Permission to filter currencies by or NULL to bypass check
	 * @param	\IPS\Member		$member		Member to check permissions for or NULL for current logged in member
	 * @return	array
	 */
	public function currencies( $perm=NULL, $member=NULL )
	{
		$currencies = array();
		
		if ( $this->currencies == 0 )
		{
			return \IPS\points\Currency::roots( $perm, $member );
		}
		
		if ( $this->currencies !== NULL )
		{
			foreach( explode( ',', $this->currencies ) as $currency_id )
			{
				try
				{
					$currency = \IPS\points\Currency::load( $currency_id );
					if ( $perm === NULL or $currency->can( $perm, $member ) )
					{
						$currencies[] = $currency;
					}
				}
				catch( \OutOfRangeException $e ) { }
			}
		}
		
		return $currencies;
	}
	
	/**
	 * Get balances for a member
	 *
	 * @param	\IPS\Member		$member			Member to get balances for
	 * @return	\IPS\points\Balance
	 */
	public function balances( $member )
	{
		$balances = array();
		
		foreach( $this->currencies( 'use', $member ) as $currency )
		{
			$balances[] = \IPS\points\Balance::get( $member, $currency, $this );
		}
		
		return $balances;
	}
	
	/**
	 * Create a table of balances
	 *
	 * @param	int			$bank_id	Bank to show balances from
	 * @param	\IPS\Http\Url		$url		Url that the table is being displayed on
	 * @return	\IPS\Helpers\Table\Db
	 */
	public static function balancesTable( $bank_id, $url )
	{
		/* Create the table */
		$table = new \IPS\Helpers\Table\Db( 'points_balances', $url, array( array( 'balance_bank_id=?', $bank_id ) ) );
		
		$table->include = array( 'balance_member_id', 'balance_currency_id', 'balance_amount' );
		$table->langPrefix = 'table_';
		$table->sortBy = \IPS\Request::i()->sortby ?: 'balance_amount';
		$table->sortDirection = \IPS\Request::i()->sortdirection ?: 'DESC';
		$table->noSort = array( 'balance_member_id', 'balance_currency_id' );
		
		$currencies = $bank_id ? \IPS\points\Bank::load( $bank_id )->currencies() : \IPS\points\Currency::roots();
		
		/* Create filters */
		foreach( array_reverse( $currencies ) as $currency )
		{
			$filter_id = $currency::$titleLangPrefix . $currency->id;
			$table->filters[ $filter_id ] = array( 'balance_currency_id=?', $currency->id );
		}
		
		$table->quickSearch = function( $val )
		{
			$matches = iterator_to_array( \IPS\Db::i()->select( 'member_id', 'core_members', array( 'name LIKE ?', '%%' . $val . '%%' ) ) );
			if ( count( $matches ) )
			{
				return \IPS\Db::i()->in( 'balance_member_id', $matches );
			}
			else
			{
				return '1=0';
			}
		};

		$table->parsers = array
		(
			'balance_member_id' => function( $val, $row )
			{
				try
				{
					$member = \IPS\Member::load( $val );
					return \IPS\Theme::i()->getTemplate( 'global', 'core' )->userPhoto( $member, 'tiny' ) . " <a style='margin-left:10px' href='" . \IPS\Http\Url::internal( 'app=core&module=members&controller=members&do=edit&id=' ) . $member->member_id . "'>" . htmlentities( $member->name, \IPS\HTMLENTITIES, 'UTF-8', FALSE ) . "</a>";
				}
				catch( \OutOfRangeException $e )
				{
					return $val;
				}
			},
			'balance_currency_id' => function( $val, $row )
			{
				try
				{
					$currency = \IPS\points\Currency::load( $val );
					return $currency->_title;
				}
				catch( \OutOfRangeException $e )
				{
					return $val;
				}				
			},
			'balance_amount' => function( $val, $row )
			{
				try
				{
					$currency = \IPS\points\Currency::load( $row[ 'balance_currency_id' ] );
					return $currency->display( $val );
				}
				catch( \OutOfRangeException $e )
				{
					return $val;
				}
			}
		);
		
		$table->rootButtons = array
		(
			'adjust' => array
			(
				'icon'		=> 'institution',
				'title'		=> 'points_add_transaction',
				'id'		=> "balance-adjustment",
				'link'		=> \IPS\Http\Url::internal( 'app=points&module=points&controller=balances&do=adjustment' )->setQueryString( 'bank_id', $bank_id ),
				'data'		=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack( 'points_add_transaction' ) ),						
			),
		);
		
		$table->rowButtons = function( $row )
		{	
			$buttons = array();
			
			$buttons[ 'transactions' ] = array
			(
				'icon'		=> 'search',
				'title'		=> 'points_view_transactions',
				'id'		=> "{$row['balance_id']}-transactions",
				'link'		=> \IPS\Http\Url::internal( 'app=points&module=points&controller=balances&do=transactions' )->setQueryString( array( 'balance_id' => $row[ 'balance_id' ] ) ),
				'data'		=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack( 'points_transactions' ) ),
			);
			
			$buttons[ 'adjust' ] = array
			(
				'icon'		=> 'calculator',
				'title'		=> 'points_adjust_balance',
				'id'		=> "{$row['balance_id']}-adjustment",
				'link'		=> \IPS\Http\Url::internal( 'app=points&module=points&controller=balances&do=adjustment' )->setQueryString( array( 'balance_id' => $row[ 'balance_id' ] ) ),
				'data'		=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack( 'points_balance_adjustment' ) ),			
			);
			
			$buttons[ 'delete' ] = array
			(
				'icon'		=> 'trash',
				'title'		=> 'points_delete_balance',
				'id'		=> "{$row['balance_id']}-delete",
				'link'		=> \IPS\Http\Url::internal( 'app=points&module=points&controller=balances&do=delete' )->setQueryString( array( 'balance_id' => $row[ 'balance_id' ] ) )->csrf(),
				'data'		=> array( 'delete' => '' ),			
			);			
			
			return $buttons;
		};
		
		return $table;
	}
	
	/**
	 * Delete
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		
		// Send to task to delete balances
		\IPS\Task::queue( 'points', 'RemoveBank', array( 'bank_id' => $this->id ) );
		
		\IPS\Lang::deleteCustom( 'points', 'bank_description_' . $this->id );
	}
	
}
