<?php
/**
 * @brief		Member Points Systems
 * @author		Kevin Carwile (http://www.linkedin.com/in/kevincarwile)
 * @copyright		(c) 2015 - Kevin Carwile
 * @package		Points
 * @since		19 Nov 2015
 */

namespace IPS\points;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Points System
 */
class _Currency extends \IPS\Node\Model implements \IPS\Node\Permissions
{

	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;
	
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'points_currencies';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'currency_';
		
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'weight';
		
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'points_currencies';
		
	/**
	 * @brief	[Node] Enabled/Disabled Column
	 */
	public static $databaseColumnEnabledDisabled = 'enabled';

	/**
	 * @brief	[Node] ACP Restrictions
	 */
	protected static $restrictions = array
	(
		'app'		=> 'points',
		'module'	=> 'points',
		'prefix'	=> 'currency_'
	);
	
	/**
	 * @brief	[Node] App for permission index
	 */
	public static $permApp = 'points';
	
	/**
	 * @brief	[Node] Type for permission index
	 */
	public static $permType = 'points_currency';
	
	/**
	 * @brief	The map of permission columns
	 */
	public static $permissionMap = array
	(
		'view'			=> 'view',
		'use'			=> 2,
		'send'			=> 3,
		'receive'		=> 4,
		'exchange_in'		=> 5,
		'exchange_out'		=> 6,
	);
	
	/**
	 * @brief	[Node] Prefix string that is automatically prepended to permission matrix language strings
	 */
	public static $permissionLangPrefix = 'points_currency_';

	/**
	 * @brief	[Node] Title prefix.  If specified, will look for a language key with "{$key}_title" as the key
	 */
	public static $titleLangPrefix = 'points_currency_';
			
	/**
	 * @brief	[Node] Description suffix.  If specified, will look for a language key with "{$titleLangPrefix}_{$id}_{$descriptionLangSuffix}" as the key
	 */
	public static $descriptionLangSuffix = '_desc';
	
	/**
	 *  Disable Copy Button
	 */	
	public $noCopyButton = TRUE;
	
	/**
	 * Form
	 */
	public function form( &$form )
	{
		$form->add( new \IPS\Helpers\Form\Translatable( 'points_currency_name', NULL, TRUE, array( 'app' => 'points', 'key' => ( $this->id ? "points_currency_{$this->id}" : NULL ) ) ) );
		$form->add( new \IPS\Helpers\Form\Translatable( 'points_currency_singular', NULL, TRUE, array( 'placeholder' => 'point', 'app' => 'points', 'key' => ( $this->id ? "points_currency_singular_{$this->id}" : NULL ) ) ) );
		$form->add( new \IPS\Helpers\Form\Translatable( 'points_currency_plural', NULL, TRUE, array( 'placeholder' => 'points', 'app' => 'points', 'key' => ( $this->id ? "points_currency_plural_{$this->id}" : NULL ) ) ) );
		$form->add( new \IPS\Helpers\Form\Text( 'currency_prefix', $this->prefix, FALSE, array( 'maxlength' => 56 ) ) );
		$form->add( new \IPS\Helpers\Form\Text( 'currency_suffix', $this->suffix, FALSE, array( 'maxlength' => 56 ) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'currency_decimals', $this->decimals !== NULL ? $this->decimals : 0, TRUE, array( 'min' => 0, 'max' => 5 ) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'currency_exchange_rate', $this->exchange_rate ?: 1, TRUE, array( 'decimals' => 2, 'min' => 0.01 ) ) );
		$form->add( new \IPS\Helpers\Form\Translatable( 'points_currency_description', NULL, FALSE, array
		(
			'app'		=> 'points',
			'key'		=> ( $this->id ? "points_currency_{$this->id}_desc" : NULL ),
			'editor'	=> array(
				'app'			=> 'points',
				'key'			=> 'Generic',
				'autoSaveKey'		=> ( $this->id ? "currency-desc-{$this->id}" : "currency-new-desc" ),
				'attachIds'		=> $this->id ? array( $this->id, NULL, 'points_currency_description' ) : NULL, 
				'minimize'		=> 'currency_desc_placeholder'
			)
		) ) );
		
		parent::form( $form );
	}
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function saveForm( $values )
	{
		$is_new = $this->_new;
		
		$points_name = $values[ 'points_currency_name' ];
		$points_description = $values[ 'points_currency_description' ];
		$points_singular = $values[ 'points_currency_singular' ];
		$points_plural = $values[ 'points_currency_plural' ];
		
		unset( $values[ 'points_currency_name' ] );
		unset( $values[ 'points_currency_singular' ] );
		unset( $values[ 'points_currency_plural' ] );
		unset( $values[ 'points_currency_description' ] );
		
		parent::saveForm( $values );
		
		/* Custom language fields */
		\IPS\Lang::saveCustom( 'points', "points_currency_{$this->id}", $points_name );
		\IPS\Lang::saveCustom( 'points', "points_currency_{$this->id}_desc", $points_description );
		\IPS\Lang::saveCustom( 'points', "points_currency_singular_{$this->id}", $points_singular );
		\IPS\Lang::saveCustom( 'points', "points_currency_plural_{$this->id}", $points_plural );
		
		/* Claim attachments */
		if ( $is_new )
		{
			\IPS\File::claimAttachments( 'currency-new-desc', $this->id, NULL, 'description', TRUE );
		}
		else
		{
			\IPS\File::claimAttachments( 'currency-desc-' . $this->id, $this->id, NULL, 'description', TRUE );
		}
		
	}

	/**
	 * [Node] Does the currently logged in user have permission to copy this node?
	 *
	 * @return	bool
	 */
	public function canCopy()
	{
		return FALSE;
	}
	
	/** 
	 * Round an amount to currency precision
	 *
	 * @param	int|float	$amount		Amount to round
	 * @return	float
	 */
	public function round( $amount )
	{
		$amount = round( $amount, $this->decimals );		
		return $amount;
	}
	
	/** 
	 * Return the closest currency value that doesn't exceed the given amount
	 *
	 * @param	int|float	$amount		Amount to round
	 * @return	float
	 */
	public function smallest( $amount )
	{	
		if ( $this->decimals > 0 )
		{
			$amount *= pow( 10, $this->decimals );
			$amount = floor( round( $amount ) );
			$amount /= pow( 10, $this->decimals );
		}
		else
		{
			$amount = floor( $amount );
		}
				
		return $amount;
	}
	
	/** 
	 * Return the closest currency value that the given amount fits into
	 *
	 * @param	int|float	$amount		Amount to round
	 * @return	float
	 */
	public function largest( $amount )
	{
		if ( $this->decimals > 0 )
		{
			$amount *= pow( 10, $this->decimals );
			$amount = ceil( round( $amount ) );
			$amount /= pow( 10, $this->decimals );
		}
		else
		{
			$amount = ceil( $amount );
		}
		
		return $amount;
	}
	
	/**
	 * Exchange an amount to another currency
	 * 
	 * @param	int|float		$amount		The amount of this currency to exchange
	 * @param	\IPS\points\Currency	$currency	The exchange currency
	 */
	public function exchange( $amount, $currency )
	{
		$exchanged_amount = $currency->smallest( $amount * ( $currency->exchange_rate / $this->exchange_rate ) );
		
		return $exchanged_amount;
	}
	
	/**
	 * Display format
	 * 
	 * @param	int|float	$amount		The amount to format
	 * @return	string
	 */
	public function display( $amount )
	{
		$neg = '';
		
		if ( $amount < 0 )
		{
			$amount *= -1;
			$neg = '-';
		}
		
		return $neg . $this->prefix . $this->format( $amount ) . $this->suffix;
	}
	
	/**
	 * Number format
	 *
	 * @param	int|float		$amount		The number to format
	 * @return	string
	 */
	public function format( $amount )
	{
		return number_format( $this->round( $amount ), $this->decimals, \IPS\Member::loggedIn()->language()->locale['decimal_point'], \IPS\Member::loggedIn()->language()->locale['thousands_sep'] );
	}
	
	/**
	 * Singular Language
	 *
	 * @param	\IPS\Member 		$member			Member to get language for
	 * @return	string
	 */
	public function singular( \IPS\Member $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		$singular = $member->language()->addToStack( 'points_currency_singular_' . $this->id );
		
		$member->language()->parseOutputForDisplay( $singular );
		
		return $singular;
	}
	
	/**
	 * Plural Language
	 *
	 * @param	\IPS\Member 		$member			Member to get language for
	 * @return	string
	 */
	public function plural( \IPS\Member $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		$plural = $member->language()->addToStack( 'points_currency_plural_' . $this->id );
		
		$member->language()->parseOutputForDisplay( $plural );
		
		return $plural;
	}
	
	/**
	 * Pluralized Language
	 *
	 * @param	int|float		$amount		Amount which needs pluralized
	 * @param	\IPS\Member 		$member		Member to get language for
	 * @return 	string
	 */
	public function pluralized( $amount, \IPS\Member $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		
		if ( $amount == 1 )
		{
			return $this->singular( $member );
		}
		else
		{
			return $this->plural( $member );
		}
	}
	
	/**
	 * Get amount of currency circulating
	 *
	 * @param	bool		$recount		Force a recount
	 * @param	int		$group_id		Return count for a specific member group
	 */
	public function circulating( $recount=FALSE, $group_id=NULL )
	{
		$recountKey 	= 'recounted_currency_circulating_' . $this->id;
		$circulatingKey	= 'currency_circulating_' . $this->id;
		
		if ( ! $recount )
		{
			try
			{
				$last_recount = \IPS\Data\Store::i()->$recountKey;
				
				/* Automatically recount based on user ttl setting */
				if ( $last_recount < time() - ( 60 * \IPS\Settings::i()->points_circulating_ttl ) )
				{
					$total = \IPS\Data\Store::i()->$circulatingKey;
					if ( $group_id )
					{
						if ( isset( $total[ $group_id ] ) )
						{
							return $this->round( $total[ $group_id ] );
						}
						
						return 0;
					}
					
					return $this->round( array_sum( $total ) );
				}
			}
			catch( \Exception $e ) { exit( 'yuk' ); }
		}
		
		$sums = \IPS\Db::i()->select( 'member_group_id, SUM(balance_amount) as total', 'points_balances', array( 'balance_currency_id=?', $this->id ), NULL, NULL, 'member_group_id' )->join( 'core_members', array( 'member_id=balance_member_id' ) );
		$total = array();
		
		foreach( $sums as $sum )
		{
			$total[ $sum[ 'member_group_id' ] ] = $sum[ 'total' ];
		}
		
		\IPS\Data\Store::i()->$recountKey = time();
		\IPS\Data\Store::i()->$circulatingKey = $total;
		
		if ( $group_id )
		{
			if ( isset( $total[ $group_id ] ) )
			{
				return $this->round( $total[ $group_id ] );
			}
			
			return 0;
		}
		
		return $this->round( array_sum( $total ) );
	}
	
	/**
	 * Get a chart of the distribution of this currency between member groups
	 *
	 * @return	\IPS\Helpers\Chart
	 */
	public function groupDistributionChart()
	{
		$chart = new \IPS\points\Chart;
		
		$chart->addHeader( \IPS\Member::loggedIn()->language()->addToStack( 'points_chart_member_group' ), 'string' );
		$chart->addHeader( \IPS\Member::loggedIn()->language()->addToStack( 'points_chart_net_worth' ), 'number' );
		
		foreach( \IPS\Member\Group::groups( TRUE, FALSE ) as $group )
		{	
			$worth = $this->circulating( FALSE, $group->g_id );
			if ( $worth !== 0 )
			{
				$chart->addRow( array( $group->name, $this->round( $worth ) ) );
			}
		}
		
		return $chart;		
	}
	
	/** 
	 * Get a chart of the total net worth for a member over a period of time
	 *
	 * @param	\IPS\Member|NULL 		$member			Member to get total balance for
	 * @param	string				$history		Period of history to show on chart
	 * @param	string				$plotInterval		Period between each plotted point
	 * @param	string				$dateFormat		Date format to show on chart
	 * @param	\IPS\DateTime			$date			The ending date/time of the chart
	 * @return	\IPS\Helpers\Chart
	 */
	public function netWorthChart( \IPS\Member $member=NULL, $history='P7D', $plotInterval='P1D', $dateFormat='n/j/Y', \IPS\DateTime $date=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		$chart = new \IPS\points\Chart;
		
		$chart->addHeader( $member->language()->addToStack( 'points_chart_date' ), 'string' );
		$chart->addHeader( $member->language()->addToStack( 'points_chart_net_worth' ), 'number' );
		
		$date = $date ?: ( $member->timezone ? new \IPS\DateTime( 'tomorrow -1 second', new \DateTimeZone( $member->timezone ) ) : new \IPS\DateTime( 'tomorrow -1 second' ) );
		$endAt = $date->getTimestamp();
		
		$date->sub( new \DateInterval( $history ) );

		while ( $date->getTimestamp() <= $endAt )
		{	
			$chart->addRow( array( $date->format( $dateFormat ), $this->round( $this->netWorthAsOf( $date, $member ) ) ) );
			$date->add( new \DateInterval( $plotInterval ) );
		}
		
		return $chart;
	}
	
	/**
	 * Get net worth as of a certain date/time
	 *
	 * @param	\IPS\DateTime		$date			The date/time to retrieve the balance for
	 * @param	\IPS\Member		$member			The member to retrieve the balance for
	 * @return	int|float
	 */
	public function netWorthAsOf( \IPS\DateTime $date=NULL, \IPS\Member $member=NULL )
	{
		$date = $date ?: \IPS\DateTime::create();
		$member = $member ?: \IPS\Member::loggedIn();
		
		$available_banks = array();
		foreach( \IPS\points\Bank::roots( 'use', $member ) as $bank )
		{
			if ( $bank->canUse( $this, $member ) )
			{
				$available_banks[] = $bank->id;
			}
		}
		
		try
		{
			$transactions = \IPS\Db::i()->select( 'MAX(transaction_id) as transaction_id', 'points_transactions', array( 'balance_currency_id=? AND balance_member_id=? AND transaction_timestamp <=? AND ( ' . \IPS\Db::i()->in( 'balance_bank_id', $available_banks ) . ' OR balance_bank_id=0 )', $this->id, $member->member_id, $date->getTimestamp() ), 'transaction_id DESC', NULL, 'balance_bank_id' )
			->join( 'points_balances', array( 'balance_id=transaction_balance_id' ) );
			
			if ( $transaction_ids = iterator_to_array( $transactions ) )
			{
				$balances = iterator_to_array( \IPS\Db::i()->select( 'transaction_balance', 'points_transactions', array( \IPS\Db::i()->in( 'transaction_id', $transaction_ids ) ) ) );				
				return $this->smallest( array_sum( $balances ) );
			}
		}
		catch( \UnderflowException $e ) { }
		
		return 0;
	}

	/**
	 * Delete
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		
		// Send to task to delete balances
		\IPS\Task::queue( 'points', 'RemoveCurrency', array( 'currency_id' => $this->id ) );
		
		\IPS\Lang::deleteCustom( 'points', 'points_currency_' . $this->id );
		\IPS\Lang::deleteCustom( 'points', 'points_currency_' . $this->id . '_desc' );
		\IPS\Lang::deleteCustom( 'points', 'points_currency_singular_' . $this->id );
		\IPS\Lang::deleteCustom( 'points', 'points_currency_plural_' . $this->id );
		
		$recountKey 	= 'recounted_currency_circulating_' . $this->id;
		$circulatingKey	= 'currency_circulating_' . $this->id;
		unset( \IPS\Data\Store::i()->$recountKey );
		unset( \IPS\Data\Store::i()->$circulatingKey );

	}
}
