<?php
/**
 * @brief		Member Points Systems
 * @author		Kevin Carwile (http://www.linkedin.com/in/kevincarwile)
 * @copyright		(c) 2015 - Kevin Carwile
 * @package		Points
 * @since		19 Nov 2015
 */

namespace IPS\points;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Points Economy
 */
abstract class _Economy
{
	public $economy;

	/**
	 * Economy Alerts
	 * -------------------
	 * Used to supply alerts related to this economy
	 *
	 * @param	\IPS\Member		$member			The member to get alerts for
	 * @return	array
	 * (
	 * 	array(
	 * 		'type' => 'info', 		// ipsMessage type
	 * 		'title'	=> '',			// Title of alert
	 * 		'message' => '',		// Alert HTML Content
	 * 	)
	 * )
	 */
	abstract public function alerts( \IPS\Member $member );
	
	/**
	 * Economy Summary
	 * ------------------
	 * This summary is used in the side column of the dashboard
	 *
	 * @param	\IPS\Member		$member			The member to display the summary for
	 * @return	string						HTML Output
	 */
	abstract public function dashSummary( \IPS\Member $member );
	
	/**
	 * Economy Portal
	 * --------------------
	 * Output detailed information in the main area of the dashboard related to this economy
	 *
	 * @param	\IPS\Member		$member			The member to display the portal for
	 * @return	string						HTML Output
	 */
	abstract public function dashPortal( \IPS\Member $member );
	
	/**
	 * Economy Member Profile Output
	 * --------------------
	 * Output overview information for this economy on the main economy homepage
	 *
	 * @param	\IPS\Member		$member			The member to display content for
	 * @return	string						HTML Output
	 */
	abstract public function profileContent( \IPS\Member $member );	
	

}
