<?php
/**
 * @brief		ExpireTransactions Task
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	points
 * @since		03 Jan 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\points\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * ExpireTransactions Task
 */
class _ExpireTransactions extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{
		/* Expire up to 50 credits at a time */
		foreach( new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( '*', 'points_credits', array( 'credit_expiration<? AND credit_transaction_id=0', time() ), 'credit_expiration ASC', 50 ), 'IPS\points\Transaction\Credit' ) as $credit )
		{
			$credit->expire();
		}
	
		/* Issue up to 50 debits at a time */
		foreach( new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( '*', 'points_debits', array( 'debit_expiration<? AND debit_transaction_id=0', time() ), 'debit_expiration ASC', 50 ), 'IPS\points\Transaction\Debit' ) as $debit )
		{
			$debit->issue( $debit->message, array(), FALSE );
		}	
	
		return NULL;
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}