<?php
/**
 * @brief		myBalances Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	points
 * @since		03 Jan 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\points\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * myBalances Widget
 */
class _myBalances extends \IPS\Widget
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'myBalances';
	
	/**
	 * @brief	App
	 */
	public $app = 'points';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
		$configuration = $this->configuration;
		
  		$form = $form ?: new \IPS\Helpers\Form;		
		$form->add( new \IPS\Helpers\Form\Checkbox( 'points_unbanked', isset( $configuration[ 'points_unbanked' ] ) ? $configuration[ 'points_unbanked' ] : 1, FALSE ) );
		$form->add( new \IPS\Helpers\Form\Node( 'points_myBalances_banks', $configuration[ 'points_myBalances_banks' ] ?: 0, TRUE, array( 'class' => 'IPS\points\Bank', 'multiple' => TRUE, 'zeroVal' => 'all_banks' ) ) );
 
		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
		$banks = array();
		
		if ( $values[ 'points_myBalances_banks' ] != 0 )
		{
			foreach( (array) $values[ 'points_myBalances_banks' ] as $bank )
			{
				if ( $bank instanceof \IPS\points\Bank )
				{
					$banks[] = $bank->id;
				}
			}
			$values[ 'points_myBalances_banks' ] = $banks;
		}
		
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		if ( \IPS\Member::loggedIn()->member_id )
		{
			$configuration = $this->configuration;
			$member = \IPS\Member::loggedIn();
			$summary = '';
			
			if ( $configuration[ 'points_unbanked' ] )
			{
				$unbanked = new \IPS\points\Account\Bank( $member );
				$unbanked->icon = 'money';
				$summary .= $unbanked;			
			}
			
			foreach( \IPS\points\Bank::roots( 'use', $member ) as $bank )
			{
				if ( $configuration[ 'points_myBalances_banks' ] == 0 or in_array( $bank->id, (array) $configuration[ 'points_myBalances_banks' ] ) )
				{
					$summary .= new \IPS\points\Account\Bank( $member, $bank );
				}
			}
			
			if ( $summary )
			{
				return $this->output( $summary );
			}
		}
		
		return '';
	}
}